# -*- coding: utf-8 -*-

import json
from os.path import dirname, abspath, join

__all__ = ['load_codes', 'language_codes']


def load_codes(file):
    '''
    Opens up file located under the etc directory containing language
    codes and prints them out
    '''
    filepath = abspath(join(dirname(__file__), 'etc', file))
    with open(filepath, 'rt') as data:
        return json.load(data)


def language_codes(lang):
    '''
    Returns a dict corresponding to the language codes available for source lang
    '''
    table = {}
    codes = load_codes('supported_translations.json')[lang]
    for code in codes:
        table[code['language']] = code['name']
    return table

def print_table(lang):
    '''
    Generates a formatted table of language codes
    '''
    table = {}
    codes = load_codes('supported_translations.json')[lang]
    for code in codes:
        print(
            u'{language:<8} {name:\u3000<20}'.format(
                name=code['name'],
                language=code['language'])
        )
    return



# -*- coding: utf-8 -*-
import sys
from argparse import ArgumentParser

from .__version__ import __version__, __build__
from .translator import (text_sink, spooler, source)

__all__ = []

description = '''A simple command line utility for translating text
    using Google Translate.'''

epilog = '''If only 1 positional argument is specified, it will be assumed
    to be dest and source will default to english.'''


def main():
    '''
    Main Entry point for translator and argument parser
    '''
    version = ''.join([__version__, __build__])

    # Language Code Parser
    langs = ArgumentParser(add_help=False)

    # Languages
    langs.add_argument(
        '-l', '--languages',
        nargs='?',
        default=False,
        const='en',
        metavar='code',
        help='List out available languages codes')

    codes = langs.parse_known_args()

    # Parse Languages
    if codes[0].languages:
        from .languages import print_table
        print_table(codes[0].languages)
        sys.exit(0)

    # Argument Parser
    parser = ArgumentParser(
        parents=[langs],
        prog='translate',
        description=description,
        epilog=epilog)

    # Version
    parser.add_argument(
        '-v', '--version', action='version',
        version="%s v%s" % ('translate', version))

    # Source Language
    parser.add_argument(
        'source',
        help='Source language code',
        nargs='?',
        default='en')

    # Destination Language
    parser.add_argument(
        'dest',
        help='Destination language code')

    args = parser.parse_args()

    source(spooler(text_sink(args.source, args.dest)))

if __name__ == '__main__':
    sys.exit(main())



# -*- coding: utf-8 -*-
try:
    import unittest2 as unittest
except ImportError:
    import unittest

import os
import sys

from nose.tools import *

sys.path.insert(0, os.path.abspath('..'))

try:
    from languages import load_codes, language_codes
except ImportError:
    from translate import load_codes, language_codes


class TestLanguages(unittest.TestCase):

    def test_load(self):
        codes = load_codes('supported_translations.json')
        self.assertIsInstance(codes, dict)
        self.assertIsInstance(codes['ko'], list)

        ko_codes = {}
        for code in codes['ko']:
            ko_codes[code['language']] = code['name']

        self.assertEqual(ko_codes.get('en'), u'영어')

    def test_codes(self):
        table = language_codes('ko')
        self.assertEqual(table['en'], u'영어')


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
try:
    import unittest2 as unittest
except ImportError:
    import unittest

import os
import sys
from nose.tools import *

sys.path.insert(0, os.path.abspath('..'))

try:
    from translator import translator
except ImportError:
    from translate import translator


class TestTranslator(unittest.TestCase):

    def typeassert(self):
        instance = translator('en', 'en', str())
        self.assertIsInstance(instance, dict)
        self.assertIsInstance(instance['sentences'], list)
        self.assertIsInstance(instance['sentences'][0], dict)
        self.assertIsInstance(instance['sentences'][0]['trans'], str)

    def test_love(self):
        love = translator('en', 'zh-TW', 'I love you')['sentences'][0]['trans']
        self.assertEqual(love, u'我愛你')


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
"""
translator
~~~~~~~~~~

Defines the interaction with the translation service.
Since the program interfaces with the google web service, this
module deals with the client side logic of pushing the translation request
to the the server.
"""

import sys
import json
from functools import wraps

try:
    from urllib.request import urlopen, Request, quote
    from urllib.parse import urlencode
except ImportError:
    from urllib2 import urlopen, Request, quote
    from urllib import urlencode

__all__ = [
    'push_url',
    'translator',
    'coroutine',
    'text_sink',
    'spooler',
    'source'
]


def push_url(site):
    '''
    Decorator that wraps the translator method and performs
    the GET HTTP request.

    Returns a dict response object from the server containing the translated
    text and metadata of the request body
    '''

    @wraps(site)
    def connection(*args, **kwargs):
        agent = 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:27.0) Gecko/20100101 Firefox/27.0'
        headers = {'User-Agent': agent}
        url = site(*args, **kwargs)
        request = Request(url, headers=headers)

        # Make HTTP Request
        req = urlopen(request)
        req_stream = req.read().decode('utf-8')
        req.close()
        return json.loads(req_stream)

    return connection


@push_url
def translator(source, target, phrase):
    """
    Returns the url encoded string that will be pushed to the translation
    server for parsing.

    :param source: Language code for translation source
    :type source: str
    :param target: Language code that source will be translate into
    :type target: str
    :param phrase: Text body string that will be url encoded and translated
    :type phrase: str
    :returns: url
    :rtype: str

    List of acceptable language codes for source and target languages
    can be found as a JSON file in the etc directory.

    Some source languages are limited in scope of the possible target languages
    that are available.

    .. code-block:: python

        >>> import translate
        >>> translator('en', 'zh-TW', 'Hello World!')
            '你好世界！'

    """

    base = 'http://translate.google.com/translate_a/t'
    params = urlencode(
    {
        'client': 'webapp',
            'ie': 'utf-8',
            'oe': 'utf-8',
            'sl': source,
            'tl': target,
             'q': phrase,
    })
    url = '?'.join([base, params])
    return url


def coroutine(func):
    """
    Co-routine decorator primes first call to next
    """

    @wraps(func)
    def initialization(*args, **kwargs):
        start = func(*args, **kwargs)
        try:
            start.__next__()
        except AttributeError:
            start.next()
        return start

    return initialization


@coroutine
def text_sink(source, dest):
    """Coroutine end-point. Outputs text stream into translator"""
    while True:
        line = (yield)
        translation = translator(source, dest, line)['sentences']
        for line in translation:
            sys.stdout.write(line['trans'])


# Make less http requests by chunking.
# Google maxes chunks sizes around 2k characters
@coroutine
def spooler(iterable):
    """Consumes text stream and spools into larger chunk for processing"""
    try:
        while True:
            wordcount, spool = 0, str()
            while wordcount < 1500:
                stream = (yield)
                spool += stream
                wordcount += len(quote(stream).encode('utf-8'))
            else:
                iterable.send(spool)
    finally:
        iterable.send(spool)
        sys.stdout.write('\n')


def source(target):
    """Coroutine start point. Produces text stream and forwards to consumers"""

    # TODO: Implement FileIO
    if sys.stdin.isatty():
        # target.send(args.file)
        return

    for line in sys.stdin:
        target.send(line)
    target.close()



# -*- coding: utf-8 -*-
'''
A simple command line utility for translating text
using Google Translate.
'''

from .translator import *
from .languages import *
from .__version__ import __version__, __build__

__title__ = 'translate'
__author__ = 'Sang Han'
__license__ = 'Apache Software License Version 2.0'
__copyright__ = 'Sang Han 2014'



# -*- coding: utf-8 -*-
import translate

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

def read(path):
    """Build a file path from *paths* and return the contents."""
    with open(path) as f:
        return f.read()


description = '''A simple command line utility for translating text
    using Google Translate.'''
long_description = '\n\n'.join(
    [
        read('README.rst'),
        read('HISTORY.rst')
    ])

setup(
    name='py-translate',
    description=description,
    long_description=long_description,
    author='Sang Han',
    license=translate.__license__,
    url='https://github.com/jjangsangy/py-translate',
    download_url='https://github.com/jjangsangy/py-translate.git',
    author_email='jjangsangy@gmail.com',
    include_package_data=True,
    packages=['translate'],
    version=translate.__version__,
    tests_require=['nose'],
    entry_points={
        'console_scripts': [
            'translate = translate.__main__:main'
            ]
        },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Unix Shell',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Text Processing :: Linguistic',
        'Topic :: Utilities',
    ],
)
